@@ -111,17 +111,19 @@ The literals of the built-in shaders got converted into constants, following the
 answer of `this SO question <https://stackoverflow.com/questions/20936086>`_.
 """
 from functools import partial
-from typing import Any, Callable, Dict, List, Optional
+from typing import Any, Callable, Dict, List, Optional, Union
 
 from kivy.clock import Clock                                    # type: ignore
 from kivy.factory import Factory                                # type: ignore
 from kivy.graphics.instructions import RenderContext            # type: ignore # pylint: disable=no-name-in-module
-from kivy.graphics.vertex_instructions import Rectangle         # type: ignore # pylint: disable=no-name-in-module
+from kivy.graphics.vertex_instructions import (                 # type: ignore # pylint: disable=no-name-in-module
+    Ellipse, Rectangle, RoundedRectangle)
 from kivy.properties import ListProperty                        # type: ignore # pylint: disable=no-name-in-module
+
 from ae.base import UNSET                                       # type: ignore
 
 
-__version__ = '0.1.7'
+__version__ = '0.1.8'
 
 
 ShaderIdType = Dict[str, Any]                                   #: shader internal data and id
@@ -138,8 +140,9 @@ uniform vec4 tint_ink;
 
 void main(void)
 {
-  vec2 pix_pos = (frag_modelview_mat * gl_FragCoord).xy - win_pos;
+  vec2 pix_pos = (frag_modelview_mat * gl_FragCoord).xy;
   float len = length(pix_pos - center_pos);
+  pix_pos -= win_pos;
   float dis = len / max(pix_pos.x, max(pix_pos.y, max(resolution.x - pix_pos.x, resolution.y - pix_pos.y)));
   vec3 col = tint_ink.rgb;
   if (tex_col_mix != 0.0) {
@@ -169,8 +172,9 @@ const float ONE = 0.99;
 void main(void)
 {
   vec2 pix_pos = (frag_modelview_mat * gl_FragCoord).xy - win_pos;
-  float x = abs(pix_pos.x - center_pos.x);
-  float y = abs(pix_pos.y - center_pos.y - resolution.y);
+  vec2 rel_center = center_pos - win_pos;
+  float x = abs(pix_pos.x - rel_center.x);
+  float y = abs(pix_pos.y - rel_center.y - resolution.y);
 
   float m1 = x + y + cos(sin(time) * TWO) * HUNDRED + sin(x / HUNDRED) * THOUSAND;
   float m2 = y / resolution.y;
@@ -181,7 +185,7 @@ void main(void)
   float c3 = abs(sin(c2 + cos(m2 + m3 + c2) + cos(m3) + sin(x / THOUSAND)));
 
   vec4 tex = texture2D(texture0, tex_coord0);
-  float dis = TWO * distance(pix_pos, center_pos) / min(resolution.x, resolution.y);
+  float dis = TWO * distance(pix_pos, rel_center) / max(resolution.x, resolution.y);
   vec4 col = vec4(c1, c2, c3, contrast * (ONE - dis)) * tint_ink * TWO;
   col = mix(tex, col, tex_col_mix);
   gl_FragColor = vec4(col.rgb, col.a * sqrt(alpha));
@@ -195,7 +199,6 @@ uniform float tex_col_mix;
 uniform float time;
 uniform vec2 center_pos;
 uniform vec2 mouse;         // density, speed
-uniform vec2 win_pos;
 uniform vec2 resolution;
 uniform vec4 tint_ink;
 
@@ -234,7 +237,7 @@ void main() {
  const vec3 c4 = vec3(165.0/255.0, 129.0/255.0, 214.4/255.0);
  const vec3 c5 = vec3(0.12);
  const vec3 c6 = vec3(0.9);
- vec2 pix_pos = (gl_FragCoord.xy - win_pos - center_pos) / resolution.xy - vec2(0.0, 0.51);
+ vec2 pix_pos = (gl_FragCoord.xy - center_pos) / resolution.xy - vec2(0.0, 0.51);
  //This is how "packed" the smoke is in our area. Try changing 15.0 to 2.1, or something else
  vec2 p = pix_pos * (ONE + mouse.x / resolution.x * 15.0);
  //The fbm function takes p as its seed (so each pixel looks different) and time (so it shifts over time)
@@ -258,7 +261,6 @@ uniform float tex_col_mix;
 uniform float time;
 uniform vec2 center_pos;
 uniform vec2 mouse;  // intensity, granularity
-uniform vec2 win_pos;
 uniform vec2 resolution;
 uniform vec4 tint_ink;
 
@@ -268,12 +270,12 @@ uniform vec4 tint_ink;
 void main( void ) {
  float t = time*contrast + 23.01;
  // uv should be the 0-1 uv of texture...
- vec2 xy = (gl_FragCoord.xy - win_pos - center_pos) / resolution.yy; // - vec2(0.9);
+ vec2 xy = (gl_FragCoord.xy - center_pos) / resolution.yy; // - vec2(0.9);
  vec2 uv = vec2(atan(xy.y, xy.x) * 6.99999 / TAU, log(length(xy)) * (0.21 + mouse.y / resolution.y) - time * 0.21);
  vec2 p = mod(uv*TAU, TAU)-250.02;
  vec2 i = vec2(p);
  float c = 8.52;
- float intensity = 0.0015 + mouse.x / resolution.x / 333.3;  // = .005;
+ float intensity = 0.003 + mouse.x / resolution.x / 333.3;  // = .005;
 
  for (int n = 0; n < MAX_ITER; n++) {
    float t = t * (1.02 - (3.498 / float(n+1)));
@@ -309,7 +311,7 @@ const float ONE = 0.99999;
 void main(void)
 {
   vec2 pix_pos = (frag_modelview_mat * gl_FragCoord).xy - win_pos;
-  float len = length(pix_pos - center_pos);
+  float len = length(pix_pos - (center_pos - win_pos));
   float col_comp = (sin(len / TEN - time * TEN) + ONE) / TEN;
   float dis = len / (TWO * min(resolution.x, resolution.y));
   vec4 col = tint_ink / vec4(col_comp, col_comp, col_comp, dis / (ONE / TEN + contrast)) / TEN;
@@ -328,25 +330,26 @@ uniform float tex_col_mix;
 uniform float time;
 uniform vec2 center_pos;
 uniform vec2 mouse;         // off1, off2
-uniform vec2 win_pos;
 uniform vec2 resolution;
 uniform vec4 tint_ink;
 
+const float XIS = 0.6;
 const float ONE = 0.99999999999;
 const float TWO = 1.99999999998;
+const float SIX = 6.0;
 
 void main(void){
- vec2 centered_coord = (TWO * (gl_FragCoord.xy - win_pos - center_pos) - resolution) / resolution.y;
+ vec2 centered_coord = (TWO * (gl_FragCoord.xy - center_pos) - resolution) / resolution.y;
  centered_coord += vec2(resolution.x / resolution.y, ONE);
- centered_coord.y *= dot(centered_coord,centered_coord);
+ centered_coord.y *= dot(centered_coord, centered_coord);
  float dist_from_center = length(centered_coord);
  float dist_from_center_y = length(centered_coord.y);
- float u = 6.0/dist_from_center_y + time * 3.999;
- float v = (10.2/dist_from_center_y) * centered_coord.x;
- float grid = (ONE-pow(sin(u)+ONE, 0.6) + (ONE-pow(sin(v)+ONE, 0.6)))*dist_from_center_y*30.0*(0.03+contrast);
- float off1 = sin(fract(time*0.48)*6.27+dist_from_center*5.0001)*mouse.x/resolution.x;
- float off2 = sin(fract(time*0.48)*6.27+dist_from_center_y*12.0)*mouse.y/resolution.y;
- vec3 col = vec3(grid) * vec3(tint_ink.r*off1,tint_ink.g,tint_ink.b*off2);
+ float u = SIX / dist_from_center_y + time * SIX;
+ float v = (SIX / dist_from_center_y) * centered_coord.x;
+ float grid = (ONE - pow(sin(u) + ONE, XIS) + (ONE - pow(sin(v) + ONE, XIS))) * dist_from_center_y * contrast * 6000.0;
+ float off1 = sin((fract(time / SIX) + dist_from_center) * SIX) * (XIS + mouse.x / resolution.x);
+ float off2 = sin((fract(time / SIX) + dist_from_center_y * TWO) * SIX) * (XIS + mouse.y / resolution.y);
+ vec3 col = vec3(grid) * vec3(tint_ink.r * off1, tint_ink.g * off1 * off2 * TWO, tint_ink.b * off2);
  if (tex_col_mix != 0.0) {
   vec4 tex = texture2D(texture0, tex_coord0);
   col = mix(tex.rgb, col, tex_col_mix);
@@ -358,7 +361,10 @@ void main(void){
 BUILT_IN_SHADERS = dict(
     circled_alpha=CIRCLED_ALPHA_SHADER_CODE, colored_smoke=COLORED_SMOKE_SHADER_CODE, fire_storm=FIRE_STORM_SHADER_CODE,
     plasma_hearts=PLASMA_HEARTS_SHADER_CODE, plunge_waves=PLUNGE_WAVES_SHADER_CODE, worm_whole=WORM_WHOLE_SHADER_CODE)
-""" dict of built-in shader codes. """
+""" dict of built-in shader code blocks - key specifies shader in :paramref:`~ShadersMixin.add_shader.shader_code`. """
+
+RENDER_SHAPES = (Ellipse, Rectangle, RoundedRectangle)
+""" supported render shapes, specified in :paramref:`~ShadersMixin.add_shader.render_shape`. """
 
 
 class ShadersMixin:
@@ -382,54 +388,26 @@ class ShadersMixin:
     _pos_fbind_uid: int = 0
     _size_fbind_uid: int = 0
 
-    def on_added_shaders(self, *_args):
-        """ added_shaders list property changed event handler. """
-        self._update_shaders()
-
-    def on_parent(self, *_args):
-        """ parent changed event handler. """
-        self._update_shaders()
-
-    def _update_shaders(self):
-        shaders_active = self.added_shaders and self.parent
-
-        for shader in reversed(self.started_shaders):
-            self.stop_shader(shader, stop=False)
-
-        bound = self._pos_fbind_uid
-        if shaders_active:
-            for kwargs in self.added_shaders:
-                if kwargs['mode'] != 'stop':
-                    kwargs['play_error'] = self.play_shader(kwargs)
-            if not bound:
-                self._pos_fbind_uid = self.fbind('pos', self._pos_changed)  # STRANGE: on_ event not bind-able -> ret==0
-                self._size_fbind_uid = self.fbind('size', self._size_changed)
-                self._pos_changed()
-                self._size_changed()
-
-        elif bound:
-            self.unbind_uid('pos', self._pos_fbind_uid)
-            self.unbind_uid('size', self._size_fbind_uid)
-            self._pos_fbind_uid = 0
-            self._size_fbind_uid = 0
-
-    def add_shader(self, add_to: str = '', mode: str = 'play',
+    def add_shader(self, add_to: str = '',
                    shader_code: str = PLASMA_HEARTS_SHADER_CODE, shader_file: str = "",
                    start_time: Optional[float] = 0.0, update_freq: float = 30.0,
+                   is_running: bool = True, render_shape: Union[Any, str] = Rectangle,
                    **glsl_dyn_args) -> ShaderIdType:
         """ create new render context canvas and add it.
 
         :param add_to:          '' to add to current canvas, 'before' and 'after' to the before/after canvas of
                                 the widget instance mixed-into. If the canvas does not exist then the shaders
                                 render context will be set as a current canvas.
-        :param mode:            shader mode, either 'play', 'stop' or 'pause'.
-        :param shader_code:     fragment shader code block (will be ignored if :paramref:`.shader_file` is not empty).
+        :param shader_code:     fragment shader code block or key of BUILT_IN_SHADERS dict if first character is '='.
+                                Note that this argument will be ignored if :paramref:`.shader_file` is not empty.
         :param shader_file:     filename with the glsl shader code (with “–VERTEX” or “–FRAGMENT” sections) to load.
         :param start_time:      base/start time. Passing the default value zero is syncing the `time` glsl parameter
                                 of this renderer with :meth:`kivy.clock.Clock.get_boottime()`.
                                 Pass None for to initialize this argument to the current Clock boot time; this
                                 way the `time` glsl argument will start by zero.
         :param update_freq:     shader/renderer update frequency. Pass 0.0 for to disable creation of an update timer.
+        :param is_running:      pass False to not run the shader. To start it later set shader_id['is_running'] to True.
+                                (mainly for internal use to refresh only running shaders).
         :param glsl_dyn_args:   extra/user dynamic shader parameters, depending on the used shader code. The keys
                                 of this dict are the names of the corresponding glsl input variables in your shader
                                 code. The built-in shaders (provided by this module) providing the following glsl
@@ -464,13 +442,63 @@ class ShadersMixin:
         if 'tint_ink' not in glsl_dyn_args:
             glsl_dyn_args['tint_ink'] = (0.546, 0.546, 0.546, 1.0)    # colors * shader_code.TWO ~= (1.0, 1.0, 1.0)
 
-        shader_id = dict(add_to=add_to, mode=mode, shader_code=shader_code, shader_file=shader_file,
+        shader_id = dict(add_to=add_to, is_running=is_running, render_shape=render_shape,
+                         shader_code=shader_code, shader_file=shader_file,
                          start_time=start_time, update_freq=update_freq, glsl_dyn_args=glsl_dyn_args)
 
         self.added_shaders.append(shader_id)
 
         return shader_id
 
+    def _compile_shader(self, shader_id: ShaderIdType) -> RenderContext:
+        """ try to compile glsl shader file/code - raise ValueError if compilation failed.
+
+        :param shader_id:       shader id (internal data dict with either `shader_file` (preference) or
+                                `shader_code` key representing the shader code to use).
+        :return:                kivy/glsl render context with the compiled shader attached to it.
+        """
+        ren_ctx = RenderContext(use_parent_modelview=True, use_parent_projection=True,
+                                use_parent_frag_modelview=True)
+        with ren_ctx:
+            shape = shader_id['render_shape']
+            if isinstance(shape, str):
+                for sha in RENDER_SHAPES:
+                    if sha.__name__ == shape:
+                        shape = sha
+                        break
+                else:
+                    ValueError(f"render shape {shape} is not supported")
+            shader_id['rectangle'] = shape(pos=self.pos, size=self.size)
+
+        shader = ren_ctx.shader
+        try:
+            shader_file = shader_id['shader_file']
+            if shader_file:
+                old_value = shader.source
+                shader.source = shader_file
+            else:
+                old_value = shader.fs
+                hdr = "$HEADER$"  # Kivy header.fs file template placeholder
+                shader_code = shader_id['shader_code']
+                if shader_code[0] == '=':
+                    shader_code = BUILT_IN_SHADERS[shader_code[1:]]
+                shader.fs = ("" if hdr in shader_code else hdr) + shader_code
+            fail_reason = "" if shader.success else "shader.success is False"
+
+        except Exception as ex:
+            fail_reason = f"exception {ex}"
+            old_value = UNSET
+
+        if fail_reason:
+            if old_value is not UNSET:
+                if shader_file:
+                    shader.source = old_value
+                else:
+                    shader.fs = old_value
+            raise ValueError(f"shader compilation failed:{fail_reason} (see console output)")
+
+        return ren_ctx
+
     def del_shader(self, shader_id: ShaderIdType):
         """ remove shader_id added via add_shader.
 
@@ -492,6 +520,14 @@ class ShadersMixin:
             if 'time' in dyn_args and not callable(dyn_args['time']):
                 dyn_args['time'] += increment
 
+    def on_added_shaders(self, *_args):
+        """ added_shaders list property changed event handler. """
+        self._update_shaders()
+
+    def on_parent(self, *_args):
+        """ parent changed event handler. """
+        self._update_shaders()
+
     def play_shader(self, shader_id: ShaderIdType) -> str:
         """ create new render context canvas and add it to the widget canvas to display shader output.
 
@@ -508,36 +544,9 @@ class ShadersMixin:
         import kivy.core.window     # type: ignore # noqa: F401 # pylint: disable=unused-import, import-outside-toplevel
 
         add_to = shader_id['add_to']
-        shader_code = shader_id['shader_code']
-        shader_file = shader_id['shader_file']
         update_freq = shader_id['update_freq']
         try:
-            ren_ctx = RenderContext(use_parent_modelview=True, use_parent_projection=True,
-                                    use_parent_frag_modelview=True)
-            with ren_ctx:
-                rectangle = Rectangle(pos=self.pos, size=self.size)
-
-            shader = ren_ctx.shader
-            try:
-                if shader_file:
-                    old_value = shader.source
-                    shader.source = shader_file
-                else:
-                    old_value = shader.fs
-                    hdr = "$HEADER$"    # Kivy header.fs file template placeholder
-                    shader.fs = ("" if hdr in shader_code else hdr) + shader_code
-                fail_reason = "" if shader.success else "shader.success is False"
-            except Exception as ex:
-                fail_reason = f"exception {ex}"
-                old_value = UNSET
-            if fail_reason:
-                if old_value is not UNSET:
-                    if shader_file:
-                        shader.source = old_value
-                    else:
-                        shader.fs = old_value
-                raise ValueError(f"compilation failed:{fail_reason} (see console output)")
-
+            ren_ctx = self._compile_shader(shader_id)
             added_to = self.canvas
             if not added_to:
                 self.canvas = ren_ctx
@@ -545,8 +554,8 @@ class ShadersMixin:
                 if add_to:
                     added_to = added_to.before if add_to == 'before' else added_to.after
                 added_to.add(ren_ctx)
+            shader_id.update(added_to=added_to, render_ctx=ren_ctx)
 
-            shader_id.update(added_to=added_to, render_ctx=ren_ctx, rectangle=rectangle)
             if not self.started_shaders:
                 self.started_shaders = list()  # create attribute on this instance (class attr untouched as emtpy list)
             self.started_shaders.append(shader_id)
@@ -555,10 +564,9 @@ class ShadersMixin:
                 shader_id['timer'] = Clock.schedule_interval(partial(self._refresh_glsl, shader_id), 1 / update_freq)
 
         except (ValueError, Exception) as ex:
-            return f"ShadersMixin.add_shader_id: shader start error - {ex}"
+            return f"ShadersMixin.play_shader: shader start error - {ex}"
 
-        if shader_id['mode'] == 'stop':
-            shader_id['mode'] = 'play'
+        shader_id['is_running'] = True
 
         return ""
 
@@ -567,11 +575,6 @@ class ShadersMixin:
         for ren in self.started_shaders:
             ren['rectangle'].pos = self.to_window(*self.pos)
 
-    def _size_changed(self, *_args):
-        """ size changed event handler. """
-        for ren in self.started_shaders:
-            ren['rectangle'].size = self.size
-
     def _refresh_glsl(self, shader_id: ShaderIdType, _dt: float):
         """ timer/clock event handler for to animate and sync one canvas shader. """
         self.refresh_shader(shader_id)
@@ -588,7 +591,7 @@ class ShadersMixin:
 
         # first set the defaults for glsl fragment shader input args (uniforms)
         glsl_kwargs = shader_id['glsl_dyn_args']
-        ren_ctx['alpha'] = 0.99
+        ren_ctx['alpha'] = 0.999999
         ren_ctx['contrast'] = 0.69
         ren_ctx['win_pos'] = list(map(float, self.to_window(*self.pos)))
         ren_ctx['resolution'] = list(map(float, self.size))
@@ -608,12 +611,17 @@ class ShadersMixin:
         for shader_id in self.started_shaders:
             self.refresh_shader(shader_id)
 
-    def stop_shader(self, shader_id: ShaderIdType, stop: bool = True):
+    def _size_changed(self, *_args):
+        """ size changed event handler. """
+        for ren in self.started_shaders:
+            ren['rectangle'].size = self.size
+
+    def stop_shader(self, shader_id: ShaderIdType, set_is_running: bool = True):
         """ stop shader by removing it from started shaders.
 
         :param shader_id:       id of the shader to stop. Ignoring if the passed shader got already stopped.
-        :param stop:            pass False to prevent that the `mode` of the :paramref:`.shader_id` gets changed
-                                'stop' (for internal use to refresh all running shaders).
+        :param set_is_running:  pass False to prevent that the `is_running` of the :paramref:`.shader_id` gets changed
+                                to False (for internal use to refresh all running shaders).
         """
         shaders = self.started_shaders
         if shader_id not in shaders:
@@ -630,10 +638,33 @@ class ShadersMixin:
         if 'timer' in shader_id:
             Clock.unschedule(shader_id.pop('timer'))
 
-        if stop:
-            shader_id['mode'] = 'stop'
+        if set_is_running:
+            shader_id['is_running'] = False
 
         shaders.remove(shader_id)
 
+    def _update_shaders(self):
+        shaders_active = self.added_shaders and self.parent
+
+        for shader in reversed(self.started_shaders):
+            self.stop_shader(shader, set_is_running=False)
+
+        bound = self._pos_fbind_uid
+        if shaders_active:
+            for kwargs in self.added_shaders:
+                if kwargs['is_running']:
+                    kwargs['play_error'] = self.play_shader(kwargs)
+            if not bound:
+                self._pos_fbind_uid = self.fbind('pos', self._pos_changed)  # STRANGE: on_ event not bind-able -> ret==0
+                self._size_fbind_uid = self.fbind('size', self._size_changed)
+                self._pos_changed()
+                self._size_changed()
+
+        elif bound:
+            self.unbind_uid('pos', self._pos_fbind_uid)
+            self.unbind_uid('size', self._size_fbind_uid)
+            self._pos_fbind_uid = 0
+            self._size_fbind_uid = 0
+
 
 Factory.register('ShadersMixin', cls=ShadersMixin)
